Package: CCAFE
Type: Package
Title: Case Control Allele Frequency Estimation
Version: 1.4.0
Authors@R: c(person("Hayley", "Wolff", email = "hayley.wolff@cuanschutz.edu", role = c("cre", "aut")))
Description: Functions to reconstruct case and control AFs from summary statistics. 
    One function uses OR, NCase, NControl, and SE(log(OR)). 
    The second function uses OR, NCase, NControl, and AF for the whole sample. 
License: GPL-3
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.3.2
Roxygen: list(markdown = TRUE)
Imports: dplyr, VariantAnnotation
Depends: R (>= 4.4.0)
Suggests: testthat (>= 3.0.0), rmarkdown, markdown, knitr, tidyverse,
        DescTools, cowplot, BiocStyle, GenomicRanges,
        SummarizedExperiment, S4Vectors, IRanges
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://github.com/wolffha/CCAFE/
BugReports: https://github.com/wolffha/CCAFE/issues
biocViews: GenomeWideAssociation, ComparativeGenomics, Genetics,
        Preprocessing, SNP, Software, WholeGenome
git_url: https://git.bioconductor.org/packages/CCAFE
git_branch: RELEASE_3_23
git_last_commit: 91b20f3
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:37:38 UTC; biocbuild
Author: Hayley Wolff [cre, aut]
Maintainer: Hayley Wolff <hayley.wolff@cuanschutz.edu>
Built: R 4.6.0; ; 2026-04-29 04:36:12 UTC; unix
