# Generated by roxygen2: do not edit by hand

export(CCAFE_convertVCF)
export(CaseControl_AF)
export(CaseControl_SE)
importFrom(dplyr,filter)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
