#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @title CCAFE: Case and Control Allele Frequency Estimation
#' @name CCAFE
#' @importFrom dplyr filter mutate select left_join
#' @section Main functions:
#' - [CaseControl_AF()]
#'
#' - [CaseControl_SE()]
#' @section Utility functions:
#' - [CCAFE_convertVCF()]
#'
## usethis namespace: end
NULL
